/******************************************************************************
 * GameManager.java - Manage all of the moves that happen in a single game
 * $Id: GameManager.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: GameManager.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.7  2009/02/23 09:44:54  dick
 * Javadoc.
 *
 * Revision 1.6  2009/02/14 13:13:21  dick
 * BuckoFIBS is released under the GNU license.
 * Javadoc.
 *
 * Revision 1.5  2009/02/11 09:01:20  dick
 * Don't initialize your turn if its not your turn or you're not playing.
 *
 * Revision 1.4  2009/01/31 08:53:08  dick
 * We're moving checkers, not dice.
 *
 * Revision 1.3  2009/01/31 08:48:23  dick
 * Line contains info about and GameManager initializes YourMove.
 *
 * Revision 1.2  2009/01/31 06:08:11  dick
 * React to the replay buttons being pressed.
 *
 * Revision 1.1  2009/01/28 22:26:52  dick
 * Skeleton GameManager.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS;

import java.util.LinkedList;

import com.buckosoft.fibs.BuckoFIBS.gui.MainDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.board.ReplayToolbar;
import com.buckosoft.fibs.board.Board;
import com.buckosoft.fibs.domain.Line;

/** Track each turn in a game.
 * @author Dick Balaska
 * @since 2009/01/28
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/GameManager.java">cvs GameManager.java</a>
 */
public class GameManager {
	private LinkedList<Line>	gameMoves = new LinkedList<Line>();
	private int 				position;
	private	ReplayToolbar		replayToolbar;
	private	MainDialog			mainDialog;
	
	private	int	checkersToMove = 0;

	/** Construct a GameManager
	 */
	public GameManager() {
	}

	/** Set the reference to the ReplayToolbar so that we can turn on and off his buttons
	 * @param replayToolbar The replayToolbar
	 */
	public void setReplayToolbar(ReplayToolbar replayToolbar) {
		this.replayToolbar = replayToolbar;
	}

	/** Set the reference to the instance of MainDialog
	 * @param mainDialog The MainDialog that is running
	 */
	public	void setMainDialog(MainDialog mainDialog) {
		this.mainDialog = mainDialog;
	}

	/** Reset a game to have no moves stored.
	 */
	public void reset() {
		gameMoves.clear();
		position = 0;
		updateToolbarButtons();
	}

	/** Called when the current event is your move.
	 * @param checkersToMove The number of checkers you have to move.
	 */
	public void yourMove(int checkersToMove) {
		this.checkersToMove = checkersToMove;
	}

	/** Add this (probably board) line to the game log
	 * @param l The line to add
	 */
	public void addLine(Line l) {
		boolean move = this.gameMoves.size() == this.position;
		l.setCheckersToMove(checkersToMove);
		checkersToMove = 0;
		this.gameMoves.add(l);
		if (move) {
			this.position++;
			sendGameLine();
		}
		updateToolbarButtons();
	}

	/** Go to the first event in the game log
	 */
	public void goFirst() {
		this.position = 1;
		sendGameLine();
		updateToolbarButtons();
	}

	/** Go to the previous event in the game log
	 */
	public void goPrev() {
		if (this.position > 1)
			this.position--;
		sendGameLine();
		updateToolbarButtons();
	}

	/** Go to the next event in the game log
	 */
	public void goNext() {
		if (this.position < this.gameMoves.size())
			this.position++;
		sendGameLine();
		updateToolbarButtons();
	}

	/** Go to the last event in the game log
	 */
	public void goLast() {
		this.position = this.gameMoves.size();
		sendGameLine();
		updateToolbarButtons();
	}

	/** Undo a partial move, start your turn over.
	 */
	public void goUndo() {
		sendGameLine();
	}

	private	void sendGameLine() {
		Board board = this.mainDialog.getBoard();
		Line l = this.gameMoves.get(this.position-1);
		board.parseFibsBoard(l.getLine());
		this.mainDialog.updateBoard();
		if (board.isYouPlaying() && board.isYourMove() && position == gameMoves.size())
			this.mainDialog.yourMove(l.getCheckersToMove());
	}

	private	void updateToolbarButtons() {
		this.replayToolbar.getJButtonFirst().setEnabled(position > 1);
		this.replayToolbar.getJButtonBack().setEnabled( position > 1);
		this.replayToolbar.getJButtonNext().setEnabled( position < gameMoves.size());
		this.replayToolbar.getJButtonEnd().setEnabled(  position < gameMoves.size());
	}
}
